﻿USE [msdb]
GO
Set NoCount ON;
/*
Version: 2.1 du 20/03/2023

Ce script permet de configurer les serveurs source et cible et de déployer tous les objets requis sur les deux serveurs.
Le Workflow est:
Création des objets et collecte des données requises.
Génération du fichier de script SQL pour déploiement vers le serveur cible.
Génération de la commande de déploiement sur le serveur cible via SQLCMD en ligne de commande depuis le serveur source.

Ce script exclut les BDD qui sont déjà en LogShipping!

La documentation est ici:
https://sosql.fr/articles-techniques-sql-server/migration-de-bases-de-donnees-avec-le-log-shipping/

*/
--Déclaration des paramètres
--Begin
Declare @BackupDirectory varchar(255), @BackupNbFiles char(1), @LogShipRestoreMode bit
, @LogFilesRetentionMinutes varchar(255), @BcpUtf8 varchar(20), @BackupShare varchar(255), @SourceServer varchar(255)
, @TargetServer varchar(255), @TargetCompatibilityLevel varchar(50);
--End

--Affectation des paramètres
--Begin
	--[@BackupDirectory] Partage pour les backups (FULL, LOG) et le fichier de script généré
	Set @BackupDirectory = '\\DESKTOP-PKCD6TN\LS_Share';	
	
	--[@BackupNbFiles] Nombre de fichiers par Backup, découpe le Backup FULL en multiples fichiers (8 max recommandé)
	Set @BackupNbFiles = '4';				
	
	--[@SourceServer] Instance source
	Set @SourceServer = 'DESKTOP-PKCD6TN\SQL2016DEVCS';	
	
	--[@TargetServer] Instance cible
	Set @TargetServer = 'DESKTOP-PKCD6TN\SQL2019DEV2';	
	
	--[@TargetCompatibilityLevel] Niveau de compatibilité cible
	Set @TargetCompatibilityLevel = '150';
	
	--[@LogFilesRetentionMinutes] Rétention en minutes des backups de log
	Set @LogFilesRetentionMinutes = '15';		
	
	--[@LogShipRestoreMode] 0 = Restore log with NORECOVERY, 1 = restore log with STANDBY
	Set @LogShipRestoreMode = 0		
	
	--[@BcpUtf8]  mettre une chaine vide '' ou '-C 65001' si BCP supporte l'UTF-8
	Set @BcpUtf8 = '-C 65001';	
--End





/*========== Ne pas modifier le code ci-dessous ==========*/

Set @BackupShare = @BackupDirectory;


Declare @DefinitionText nvarchar(max);

IF Not Exists(Select top 1 1 from sys.schemas Where name = 'LogShip')
Begin
	Exec('CREATE SCHEMA [LogShip] AUTHORIZATION [dbo];')
End

IF OBJECT_ID('LogShip.LogShipObjects') is null
Begin
	Create table LogShip.LogShipObjects(DefinitionText nvarchar(max), OrderIndex int);
End
Else
Begin
	Truncate table LogShip.LogShipObjects;
End

--Cette base de test créée plus loin dans le script sert à récupérer le chemin de l'Exe du Log Shipping, elle est supprimée après dans le script
IF Exists(Select top 1 1 from sys.databases Where name = 'SoSqlLogShipTest')
Begin
	Drop DATABASE [SoSqlLogShipTest];
End

--Création et stockage dans une table des objets requis...
--La mise en table est requise pour générer le fichier de scipt de déploiement sur la cible
Set @DefinitionText = N'
IF OBJECT_ID(''LogShip.LogShipParams'') is null
Begin
	Create table LogShip.LogShipParams(ParamName varchar(50), ParamValue varchar(4000));
	Insert LogShip.LogShipParams(ParamName, ParamValue) 
	Values(''sqllogship.exe'', NULL)
		, (''DefaultJobName'', ''LS_default_Job'')
		, (''LogshipJobId'', NULL)
		, (''BackupDirectory'', '''+@BackupDirectory+''')
		, (''BackupShare'', '''+@BackupShare+''')
		, (''SourceServer'', '''+@SourceServer+''')
		, (''TargetServer'', '''+@TargetServer+''')
		, (''DbName'', ''msdb'')
		, (''SchemaName'', ''LogShip'')
		, (''DeleteRestoredBackup'', ''Y'')
		, (''LogFilesRetentionMinutes'', '''+@LogFilesRetentionMinutes+''')
		, (''BackupNbFiles'', '''+@BackupNbFiles+''')
		, (''LogShipRestoreMode'', '''+cast(@LogShipRestoreMode as varchar)+''')
		, (''ProductMajorVersion'', '''+cast(SERVERPROPERTY('ProductMajorVersion') as varchar)+''')		
End
ELSE
Begin
	Truncate table LogShip.LogShipParams;
	Insert LogShip.LogShipParams(ParamName, ParamValue) 
	Values(''sqllogship.exe'', NULL)
		, (''DefaultJobName'', ''LS_default_Job'')
		, (''LogshipJobId'', NULL)
		, (''BackupDirectory'', '''+@BackupDirectory+''')
		, (''BackupShare'', '''+@BackupShare+''')
		, (''SourceServer'', '''+@SourceServer+''')
		, (''TargetServer'', '''+@TargetServer+''')
		, (''SchemaName'', ''LogShip'')
		, (''DbName'', ''msdb'')
		, (''DeleteRestoredBackup'', ''Y'')
		, (''LogFilesRetentionMinutes'', '''+@LogFilesRetentionMinutes+''')
		, (''BackupNbFiles'', '''+@BackupNbFiles+''')
		, (''LogShipRestoreMode'', '''+cast(@LogShipRestoreMode as varchar)+''')
		, (''ProductMajorVersion'', '''+cast(SERVERPROPERTY('ProductMajorVersion') as varchar)+''')		
End'
	
	Insert into LogShip.LogShipObjects(DefinitionText, OrderIndex)
	Values(@DefinitionText, 0)

	Exec(@DefinitionText)

Set @DefinitionText = N'
IF object_id(''LogShip.InstanceLoginsRoles'') is null
Begin
	CREATE TABLE LogShip.InstanceLoginsRoles
		   (
			RowId int IDENTITY(1, 1)
			,LoginName nvarchar(max)
		   ,CreateLoginCmd nvarchar(max)
		   ,CmdType varchar(50)
		   )
End'
	Insert into LogShip.LogShipObjects(DefinitionText, OrderIndex)
	Values(@DefinitionText, 0)

	Exec(@DefinitionText)


Set @DefinitionText = N'
IF OBJECT_ID(''LogShip.DatabasesDetails'') is null
Begin
	CREATE TABLE [LogShip].[DatabasesDetails](
		[ServerInstanceName] [sysname] NULL,
		[InstanceName] [sysname] NULL,
		[DatabaseName] [sysname] NOT NULL,
		[RecoveryModel] [nvarchar](60) NULL,
		[NewRecoveryModel] [nvarchar](192) NULL,
		[OriginalRecoveryModel] [nvarchar](246) NULL,
		[VersionNum] [tinyint] NOT NULL,
		[NewVersionNum] [tinyint] NULL,
		[CompatibilityLevel] [varchar](24) NULL,
		[NewCompatibilityLevel] as (''USE [master]; ALTER DATABASE [''+[DatabaseName]+''] SET COMPATIBILITY_LEVEL = ''+Convert(varchar(20), [NewVersionNum])+'';'')
	) ON [PRIMARY]
End
ELSE
Begin
	Truncate table [LogShip].[DatabasesDetails];
End'
	Insert into LogShip.LogShipObjects(DefinitionText, OrderIndex)
	Values(@DefinitionText, 0)

	Exec(@DefinitionText)
	Insert into msdb.LogShip.DatabasesDetails([ServerInstanceName], [InstanceName], [DatabaseName], [RecoveryModel], [NewRecoveryModel], [OriginalRecoveryModel], [VersionNum], [NewVersionNum], [CompatibilityLevel])
	SELECT 
	CONVERT(sysname, SERVERPROPERTY('servername')) as ServerInstanceName, CONVERT(sysname, SERVERPROPERTY('instancename')) as InstanceName
	,name AS [DatabaseName],
	recovery_model_desc AS [RecoveryModel],
	Case 
		When recovery_model_desc <> 'FULL' Then 'USE [master]; ALTER DATABASE ['+name+'] SET RECOVERY FULL WITH NO_WAIT' Else NULL 
	End as NewRecoveryModel,
	'USE [master]; ALTER DATABASE ['+name+'] SET RECOVERY '+recovery_model_desc collate French_CI_AS+' WITH NO_WAIT' as OriginalRecoveryModel,
	compatibility_level as VersionNum,
	CONVERT(tinyint, null) as NewVersionNum,
	CompatibilityLevel = 
	CASE compatibility_level
		WHEN 65  THEN 'SQL Server 6.5'
		WHEN 70  THEN 'SQL Server 7.0'
		WHEN 80  THEN 'SQL Server 2000'
		WHEN 90  THEN 'SQL Server 2005'
		WHEN 100 THEN 'SQL Server 2008/R2'
		WHEN 110 THEN 'SQL Server 2012'
		WHEN 120 THEN 'SQL Server 2014'
		WHEN 130 THEN 'SQL Server 2016'
		WHEN 140 THEN 'SQL Server 2017'
		WHEN 150 THEN 'SQL Server 2019'
		ELSE 'new unknown - '+CONVERT(varchar(10),compatibility_level)
	END
	FROM sys.databases
	Where name not in('master', 'msdb', 'tempdb', 'model', 'SSISDB')
	And name not like 'ReportServer%'
	And name not in(
		--Exclusion des bases de données déjà en Log shipping
		SELECT primary_database as DatabaseName FROM msdb.dbo.log_shipping_primary_databases
		union all
		SELECT secondary_database as DatabaseName FROM msdb.dbo.log_shipping_secondary_databases	
	)
	And is_read_only = 0

	Update msdb.LogShip.DatabasesDetails
	Set NewVersionNum = @TargetCompatibilityLevel

Set @DefinitionText = N'	
IF  OBJECT_ID(N''LogShip.DbFilesLocation'') is not null
Begin
	DROP TABLE [LogShip].[DbFilesLocation];
End

CREATE TABLE [LogShip].[DbFilesLocation](
	[DatabaseName] [nvarchar](128) NULL,
	[Drive] [nvarchar](1) NULL,
	[GroupId] [int] NULL,
	[FILE_SIZE_MB] [decimal](12, 2) NULL,
	[SPACE_USED_MB] [decimal](12, 2) NULL,
	[FREE_SPACE_MB] [decimal](12, 2) NULL,
	[FREE_SPACE_%] [decimal](12, 2) NULL,
	[NAME] [sysname] NOT NULL,
	[FILENAME] [nvarchar](260) NOT NULL,
	[CurrentPath] [nvarchar](4000) NULL,
	[NewPath] [varchar](4000) NULL
) ON [PRIMARY]'

	Insert into LogShip.LogShipObjects(DefinitionText, OrderIndex)
	Values(@DefinitionText, 0)

	Exec(@DefinitionText)

IF OBJECT_ID('tempdb..#dbfreesize') is not null
Begin
	Drop table #dbfreesize
End
CREATE TABLE #dbfreesize(
	[DatabaseName] [nvarchar](128) NULL,
	[Drive] [nvarchar](1) NULL,
	[GroupId] int,
	[FILE_SIZE_MB] [decimal](12, 2) NULL,
	[SPACE_USED_MB] [decimal](12, 2) NULL,
	[FREE_SPACE_MB] [decimal](12, 2) NULL,
	[FREE_SPACE_%] [decimal](12, 2) NULL,
	[NAME] [sysname] NOT NULL,
	[FILENAME] [nvarchar](260) NOT NULL
)


insert into #dbfreesize([DatabaseName], [Drive], [GroupId], [FILE_SIZE_MB], [SPACE_USED_MB], [FREE_SPACE_MB], [FREE_SPACE_%], [NAME], [FILENAME]) 
Exec sp_MSforeachdb N'
Use [?];
SELECT
''?'' as DbName,
 SUBSTRING(a.FILENAME, 1, 1) Drive,
 [GroupId] = groupid,
 [FILE_SIZE_MB] = convert(decimal(12,2),
round(a.size/128.000,2)),
 [SPACE_USED_MB] = convert(decimal(12,2),
round(fileproperty(a.name,''SpaceUsed'')/128.000,2)),
 [FREE_SPACE_MB] = convert(decimal(12,2),
round((a.size-fileproperty(a.name,''SpaceUsed''))/128.000,2)) ,
 [FREE_SPACE_%] = convert(decimal(12,2),
(convert(decimal(12,2),round((a.size-fileproperty(a.name,''SpaceUsed''))/128.000,2)) 
/ convert(decimal(12,2),round(a.size/128.000,2)) * 100)),
 a.NAME, a.FILENAME
FROM dbo.sysfiles a
ORDER BY Drive, [Name]'

Insert into LogShip.DbFilesLocation([DatabaseName], [Drive], [GroupId], [FILE_SIZE_MB], [SPACE_USED_MB], [FREE_SPACE_MB], [FREE_SPACE_%], [NAME], [FILENAME], [CurrentPath], [NewPath])
Select *
,Replace(FILENAME, Reverse(LEFT(REVERSE(FILENAME), Charindex('\', REVERSE(FILENAME))-1)), '') as CurrentPath
,CONVERT(varchar(4000), null) as NewPath
from #dbfreesize
Where [DatabaseName] not in('master', 'msdb', 'tempdb', 'model', 'SSISDB', 'ReportServer', 'ReportServerTempDB')
Order by [FILE_SIZE_MB] desc



Set @DefinitionText = N'
	CREATE DATABASE [SoSqlLogShipTest];
	--Récupération du path de sqllogship.exe
	DECLARE @LS_BackupJobId	AS uniqueidentifier 
	DECLARE @LS_PrimaryId	AS uniqueidentifier 
	DECLARE @SP_Add_RetCode	As int 
	DECLARE @COMMAND varchar(4000)
	ALTER DATABASE [SoSqlLogShipTest] SET RECOVERY FULL WITH NO_WAIT;


	EXEC @SP_Add_RetCode = master.dbo.sp_add_log_shipping_primary_database 
			@database = N''SoSqlLogShipTest'' 
			,@backup_directory = N''nul:'' 
			,@backup_share = N''nul:'' 
			,@backup_job_name = N''LS_default_Job'' 
			,@backup_retention_period = 4320
			,@backup_compression = 1
			,@backup_threshold = 60 
			,@threshold_alert_enabled = 1
			,@history_retention_period = 5760 
			,@backup_job_id = @LS_BackupJobId OUTPUT 
			,@primary_id = @LS_PrimaryId OUTPUT 
			,@overwrite = 1 

	SELECT @COMMAND = Replace(Left(js.command, CHARINDEX(''-Backup'', js.command)-1), ''"'', '''')
	FROM msdb.dbo.sysjobs_view jv
	LEFT JOIN msdb.dbo.sysjobsteps js
	ON jv.job_id = js.job_id
	Where jv.name = ''LS_default_Job''
	
	Update msdb.LogShip.LogShipParams
	Set ParamValue = Ltrim(Rtrim(@COMMAND))
	Where ParamName = ''sqllogship.exe''

	Exec master..sp_delete_log_shipping_primary_database ''SoSqlLogShipTest''

	IF Exists(Select top 1 1 from sys.databases Where name = ''SoSqlLogShipTest'')
	Begin
		Drop DATABASE [SoSqlLogShipTest];
	End

	--EXEC msdb..sp_delete_job @job_name = N''LS_default_Job'' ;
	'
	
	Insert into LogShip.LogShipObjects(DefinitionText, OrderIndex)
	Values(@DefinitionText, 1)

	Exec(@DefinitionText)

IF OBJECT_ID('LogShip.sf_IsRunningJob') is null
Begin
execute dbo.sp_executesql @statement = N'
-- =============================================
-- Author:		JFP SoSql
-- Create date: 2021/11/16
-- Description: Vérifie si un job tourne
-- EX: Select [LogShip].[sf_IsRunningJob](N''SoSql_Maintenance_DIFF'');
-- =============================================
Create FUNCTION [LogShip].[sf_IsRunningJob] (@JOB_NAME SYSNAME)
RETURNS bit
AS
BEGIN
	-- Declare the return variable here
	DECLARE @ResultVar bit;
	IF EXISTS(     
				SELECT
					ja.job_id,
					j.name AS job_name,
					ja.start_execution_date,      
					ISNULL(last_executed_step_id,0)+1 AS current_executed_step_id,
					js.step_name
				FROM msdb.dbo.sysjobactivity ja 
				JOIN msdb.dbo.sysjobs j 
					ON ja.job_id = j.job_id
				JOIN msdb.dbo.sysjobsteps js
					ON ja.job_id = js.job_id
					AND ISNULL(ja.last_executed_step_id,0)+1 = js.step_id
				WHERE ja.session_id = (SELECT TOP 1 session_id FROM msdb.dbo.syssessions ORDER BY session_id DESC)
				AND start_execution_date is not null
				AND stop_execution_date is null 
				AND j.name = @JOB_NAME 
			) 
	BEGIN      
		Set @ResultVar = 1;
	END 
	ELSE 
	BEGIN 
		Set @ResultVar = 0;
	END 

	RETURN @ResultVar;
END

' End

IF (SELECT OBJECT_ID(N'[LogShip].[sf_hexadecimal]')) is null
BEGIN
execute dbo.sp_executesql @statement = N'
	Create function [LogShip].[sf_hexadecimal](@binvalue varbinary(256))
	RETURNS varchar(514)
	Begin
		Declare @charvalue varchar(514)
		Declare @i int
		Declare @length int
		Declare @hexstring char(16)

		Select @charvalue = ''0x''
		Select @i = 1
		Select @length = DATALENGTH(@binvalue)
		Select @hexstring = ''0123456789ABCDEF''

		While @i <= @length
		Begin
			Declare @tempint int
			Declare @firstint int
			Declare @secondint int

			Select @tempint = Convert(int, substring(@binvalue, @i, 1))
			Select @firstint = floor(@tempint/16)
			Select @secondint = @tempint - (@firstint*16)
			Select @charvalue = @charvalue + substring(@hexstring, @firstint+1, 1) + substring(@hexstring, @secondint+1, 1)
			Select @i = @i + 1		
		End
		Return @charvalue
	END'
End

IF OBJECT_ID('LogShip.sf_ObjectDefinition') is null
Begin
execute dbo.sp_executesql @statement = N'
Create function [LogShip].[sf_ObjectDefinition](@ObjectFullName varchar(255))
RETURNS nvarchar(max)
Begin
	Declare @String nvarchar(max);

	Set @String = N''IF OBJECT_ID(''''''+@ObjectFullName+'''''') is null''
	Set @String += NCHAR(13)+''Begin''
	Set @String += NCHAR(13)+''execute dbo.sp_executesql @statement = N''''''
	Set @String += NCHAR(13)
	Set @String += NCHAR(13)+Ltrim(Rtrim(Replace(OBJECT_DEFINITION(object_id(@ObjectFullName)), '''''''', '''''''''''')))
	Set @String += NCHAR(13)
	Set @String += NCHAR(13)+'''''' End''
	Return @String
END

' End
--Ajout de la fonction STING_SPLIT pour les versions antérieures à SQL 2016
--Begin
IF object_id('dbo.STRING_SPLIT') is null and (Select Convert(int, (Left(CONVERT(VARCHAR(128), SERVERPROPERTY ('productversion')), CHARINDEX('.', CONVERT(VARCHAR(128), SERVERPROPERTY ('productversion')))-1)))) < 13
Begin
execute dbo.sp_executesql @statement = N'
/*
    The same as STRING_SPLIT for compatibility level < 130
    https://docs.microsoft.com/en-us/sql/t-sql/functions/string-split-transact-sql?view=sql-server-ver15
	DECLARE @ID NVARCHAR(300)= ''abc,d,e,f,g'';
	select * from STRING_SPLIT(@ID,'''','''')
*/
Create FUNCTION [dbo].[STRING_SPLIT]
(
    @string    nvarchar(max),
    @delimiter nvarchar(max)
)

RETURNS TABLE AS RETURN
(
    SELECT 
      Split.a.value(''.'', ''NVARCHAR(MAX)'')                                 AS value
    FROM
    (
        SELECT CAST(''<X>''+REPLACE(@string, @delimiter, ''</X><X>'')+''</X>'' AS XML) AS String
    ) AS a
    CROSS APPLY String.nodes(''/X'') AS Split(a)
)
'
End

IF (SELECT OBJECT_ID(N'[LogShip].[up_Restore_LogShipping_DB]')) is null
BEGIN
execute dbo.sp_executesql @statement = N'
/* =============================================
-- Author:		JFP SoSql
-- Create date: 2021-15-11
-- Description:	Restore une base en log shipping
=============================================*/
CREATE PROCEDURE [LogShip].[up_Restore_LogShipping_DB]
(
@DatabaseName varchar(255)
, @BackupFile varchar(4000)
, @CompatibilityLevel varchar(1000)
, @RecoveryModel varchar(1000)
)
AS
BEGIN
	SET NOCOUNT ON;
	--Raiserror(''Exec LogShip.up_Restore_LogShipping_DB ''''%s'''', null, ''''%s'''', ''''%s'''''', 0, 1, @DatabaseName, @CompatibilityLevel, @RecoveryModel) with nowait;
	--Variables du WorkFlow
	--Begin
		Declare @LogMessage VarChar(MAX), @DateString varchar(50)
		Declare @RowId int = 0, @SynName nvarchar(128), @SynFullName nvarchar(128), @CreateSynCmd nvarchar(4000), @DropSynCmd nvarchar(4000);
		Declare @IsNullable bit = 1;
		Declare @ErrorSeverity int, @ErrorState int;
		Declare @LinkedServerDisplayName Sysname = ''LogShipLinkedServer'';
		Declare @NbRows int
	--End

	IF object_id(''tempdb..#FileListHeaders'') is not null
	Begin
		DROP TABLE #FileListHeaders
	End

	Declare @InstanceDefaultDataPath varchar(255), @InstanceDefaultLogPath varchar(255)

	IF @BackupFile is not null
	Begin
	select 
		@InstanceDefaultDataPath = Convert(varchar(255), serverproperty(''InstanceDefaultDataPath'')),
		@InstanceDefaultLogPath = Convert(varchar(255), serverproperty(''InstanceDefaultLogPath''))

		--Verification du "\" de fin
		Set @InstanceDefaultDataPath = Case When RIGHT(@InstanceDefaultDataPath, 1) <> ''\'' Then (@InstanceDefaultDataPath+''\'') Else @InstanceDefaultDataPath End
		Set @InstanceDefaultLogPath = Case When RIGHT(@InstanceDefaultLogPath, 1) <> ''\'' Then (@InstanceDefaultLogPath+''\'') Else @InstanceDefaultLogPath End;
		
		--Verifie que l''eventuel chemin est correct
		Update [LogShip].[DbFilesLocation]
		Set NewPath = Case When RIGHT(Ltrim(Rtrim(NewPath)), 1) <> ''\'' Then (Ltrim(Rtrim(NewPath))+''\'') Else NewPath End
		Where DatabaseName = @DatabaseName and NewPath is not null

		Select DatabaseName, GroupId, NAME, FILENAME, CurrentPath 
		,REVERSE(Left(REVERSE(FILENAME), CHARINDEX(''\'', REVERSE(FILENAME))-1)) as SimpleFileName
		,Case
			When NewPath is null and GroupId > 0 Then (@InstanceDefaultDataPath+REVERSE(Left(REVERSE(FILENAME), CHARINDEX(''\'', REVERSE(FILENAME))-1)))
			When NewPath is null and GroupId = 0 Then (@InstanceDefaultLogPath+REVERSE(Left(REVERSE(FILENAME), CHARINDEX(''\'', REVERSE(FILENAME))-1)))
			Else (NewPath+REVERSE(Left(REVERSE(FILENAME), CHARINDEX(''\'', REVERSE(FILENAME))-1)))
		End as NewLocation
		into #FileListHeaders
		from [LogShip].[DbFilesLocation] 
		Where DatabaseName = @DatabaseName

		Declare @RestoreCmd varchar(8000);
		
		Set @RestoreCmd = ''RESTORE DATABASE [''+@DatabaseName+''] FROM ''+@BackupFile+''  
		WITH  FILE = 1''+CHAR(13)
	

	 Set @RestoreCmd += (Select ('',  MOVE N''''''+NAME+'''''' TO N''''''+NewLocation+'''''''')
	from #FileListHeaders FOR XML PATH(''''))
	
	Set @RestoreCmd += '',  NOUNLOAD, NORECOVERY, REPLACE,  STATS = 5''
	End
	Else
	Begin
		Set @RestoreCmd = ''RESTORE DATABASE [''+@DatabaseName+''] with RECOVERY''+CHAR(13)+@RecoveryModel+CHAR(13)+@CompatibilityLevel;
	End


	Begin try
		IF DATABASEPROPERTYEX (@DatabaseName, ''Status'') = ''ONLINE''
		Begin
			Set @RestoreCmd = ''ALTER DATABASE [''+@DatabaseName+''] SET  SINGLE_USER With Rollback immediate; ''+@RestoreCmd
		End
		Raiserror(@RestoreCmd, 0, 1) with nowait;
		Exec(@RestoreCmd)
		IF @BackupFile is not null and Exists(Select top 1 1 from [LogShip].[LogShipParams] Where ParamName = ''DeleteRestoredBackup'' and ParamValue = ''Y'')
		Begin
			--Boucle sur les Backups à supprimer
			IF OBJECT_ID(''tempdb..#tmpDeleteFiles'') is not null
			Begin
				Drop table #tmpDeleteFiles;
			End
			SELECT Identity(int, 1, 1) as RowId, Ltrim(Rtrim(value)) as FileName into #tmpDeleteFiles FROM STRING_SPLIT(Replace(@BackupFile, ''disk = '', ''''), '','');

			Declare @LoopOnFiles int = 0
			While Exists(Select top 1 1 from #tmpDeleteFiles Where RowId > @LoopOnFiles Order by RowId)
			Begin
				Select top 1 @LoopOnFiles = RowId, @BackupFile = Replace(FileName, '''''''', '''') from #tmpDeleteFiles Where RowId > @LoopOnFiles Order by RowId
				Raiserror(''Suppression du ficher [%s]'', 0, 1, @BackupFile) with nowait;
				EXEC master.dbo.xp_delete_file 0, @BackupFile;
			End
		End
		Declare @DbState varchar(255), @LoopLoginroles int = 0, @LoginRoleCmd nvarchar(max), @LoginName nvarchar(max), @LoginCmdtype varchar(50);
		Select @DbState = Convert(varchar(255), DATABASEPROPERTYEX(@DatabaseName, ''Status''))
		IF @DbState = ''ONLINE''
		Begin
			While Exists(Select top 1 1 from [LogShip].[InstanceLoginsRoles] where RowId > @LoopLoginroles order by RowId asc)
			Begin
				Select top 1 @LoopLoginroles = RowId, @LoginRoleCmd = CreateLoginCmd, @LoginName = LoginName, @LoginCmdtype = CmdType from [LogShip].[InstanceLoginsRoles]  where RowId > @LoopLoginroles order by RowId asc
				Begin try
					Raiserror(''Verification des [login et roles]: [%s] for [%s]'', 0, 1, @LoginCmdtype, @LoginName) with nowait;
					Exec(@LoginRoleCmd)
				End try
				Begin catch
					Raiserror(''L''''action [%s] for [%s] n''''est pas encore applicable.'', 0, 1, @LoginCmdtype, @LoginName) with nowait;
				End catch
			End
		End
	End try
	Begin catch
		SET @LogMessage = ERROR_MESSAGE() ; 
		Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
		Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
	End catch	
End
'
End

IF (SELECT OBJECT_ID(N'[LogShip].[up_Set_LogShipping_ON]')) is null
BEGIN
execute dbo.sp_executesql @statement = N'
/*=============================================
-- Author:		JFP SoSql
-- Create date: 2021-15-11
-- Description:	Active le log shipping
-- Select * from [LogShip].[DatabasesDetails]
-- Delete [LogShip].[DatabasesDetails] Where DatabaseName = ''DbName1''
-- Exemples: 

--Active le Log Shipping sur toutes les bases de données
Exec LogShip.up_Set_LogShipping_ON

--Active le Log Shipping sur une base de données
Exec msdb.LogShip.up_Set_LogShipping_ON @LSDatabaseName = ''DemoDB2''
=============================================*/
CREATE PROCEDURE [LogShip].[up_Set_LogShipping_ON]
(
@LSDatabaseName sysname = null
)
AS
BEGIN
	SET NOCOUNT ON;
	IF OBJECT_ID(''tempdb..#LSDatabases'') is not null
	Begin
		Drop table #LSDatabases;
	End
	--Retourne la liste des bases du Log Shipping
	SELECT [DatabaseName] into #LSDatabases FROM [msdb].[LogShip].[DatabasesDetails]
	Where [DatabaseName] = Case When @LSDatabaseName is null Then [DatabaseName] Else @LSDatabaseName End
	Delete #LSDatabases
	Where [DatabaseName] in(SELECT primary_database FROM msdb.dbo.log_shipping_primary_databases)

	IF Not Exists(SELECT top 1 1 FROM #LSDatabases)
	Begin
		IF @LSDatabaseName is null
		Begin
			Raiserror(''Il n''''y a aucune base de données à ajouter au LOG SHIPPING !'', 0, 1) with nowait;
		End
		ELSE
		Begin
			Raiserror(''La base de données [%s] est déjà en LOG SHIPPING !'', 0, 1, @LSDatabaseName) with nowait;
		End
		GOTO FinalyseProcess;		
	End
	--Variables du WorkFlow
	--Begin
	Declare @LogMessage VarChar(MAX), @DateString varchar(50)
	Declare @RowId int = 0, @SynName nvarchar(128), @SynFullName nvarchar(128), @CreateSynCmd nvarchar(4000), @DropSynCmd nvarchar(4000);
	Declare @IsNullable bit = 1;
	Declare @ErrorSeverity int, @ErrorState int;
	Declare @LinkedServerDisplayName Sysname = ''LogShipLinkedServer'';
	Declare @NbRows int
	Declare @BackupDirectory varchar(255), @BackupShare varchar(255), @SourceServer varchar(255), @TargetServer varchar(255), @TargetCompatibilityLevel varchar(50), @BackupNbFiles char(1);
	Declare @EnableLogShipping bit = 0, @DatabaseName sysname;
	Declare @LS_BackupJobId	AS uniqueidentifier 
	Declare @LS_PrimaryId	AS uniqueidentifier 
	Declare @LS_AlertJobId	AS uniqueidentifier 
	Declare @SP_Add_RetCode	As int 
	Declare @LS_BackUpScheduleUID	As uniqueidentifier 
	Declare @LS_BackUpScheduleID	AS int
	Declare @LS_TargetBackUpScheduleUID	As uniqueidentifier 
	Declare @LS_TargetBackUpScheduleID	AS int
	Declare @LS_TargetBackupJobId	AS uniqueidentifier 
	Declare @LS_Secondary_CopyJobId	AS uniqueidentifier 
	Declare @OnErrorStep varchar(50)
	Declare @OnErrorNextStep varchar(50)
	Declare @LogFilesRetentionMinutes int
	Declare @BackupNbFilesInt int
	Declare @BackupToDiskString varchar(8000) = ''''
	Declare @RestoreMode bit = 0									
	--End
	
	Select @BackupShare = ParamValue from LogShip.LogShipParams Where ParamName = ''BackupShare''
	Select @BackupNbFiles = ParamValue from LogShip.LogShipParams Where ParamName = ''BackupNbFiles''
	Select @SourceServer = ParamValue from LogShip.LogShipParams Where ParamName = ''SourceServer''
	Select @TargetServer = ParamValue from LogShip.LogShipParams Where ParamName = ''TargetServer''
	Select @LogFilesRetentionMinutes = Convert(int, ParamValue) from LogShip.LogShipParams Where ParamName = ''LogFilesRetentionMinutes''
	Select @RestoreMode = Convert(bit, ParamValue) from LogShip.LogShipParams Where ParamName = ''LogShipRestoreMode''

	Set @DateString = FORMAT(GetDate(), ''yyyy-MM-dd_HH-mm-ss'')
	IF RIGHT(@BackupShare, 1) <> ''\''
	Begin
		Set @BackupShare += ''\''
	End


	IF ISNUMERIC(@BackupNbFiles) = 1
	Begin
		Set @BackupNbFilesInt = Convert(int, @BackupNbFiles)
	End
	Else
	Begin
		Raiserror(''Le paramètre @BackupNbFiles doit être une chaine numérique [1, 9] !'', 15, 16) with nowait;
		GOTO RollbackOnError;
	End

	--Définition de la commande TO DISK
	Declare @BackupFilesLoop int = 1
	While @BackupFilesLoop <= @BackupNbFilesInt
	Begin
		IF @BackupFilesLoop < @BackupNbFilesInt
		Begin
			Set @BackupToDiskString += ''disk = ''''''+@BackupShare+''$DatabaseName$_FULL_''+@DateString+''_''+Convert(varchar(10), @BackupFilesLoop)+''.bak'''','';
		End
		Else
		Begin
			Set @BackupToDiskString += ''disk = ''''''+@BackupShare+''$DatabaseName$_FULL_''+@DateString+''_''+Convert(varchar(10), @BackupFilesLoop)+''.bak'''''';
		End
		Set @BackupFilesLoop += 1
	End


	--Vérifie sur quel serveur la procedure est exécutée
	Declare @CurrentServer varchar(50)
	IF Upper(Convert(varchar(50), SERVERPROPERTY(''machinename''))) = Upper(@SourceServer) Or Upper(Convert(varchar(50), SERVERPROPERTY(''Servername''))) = Upper(@SourceServer)--, SERVERPROPERTY(''instancename'')
	Begin
		Set @CurrentServer = ''primary''
		Raiserror(''Exécution sur le serveur primaire...'', 0, 1) with nowait;
		GOTO IsPrimaryServer
	End
	Else IF Upper(Convert(varchar(50), SERVERPROPERTY(''machinename''))) = Upper(@TargetServer) Or Upper(Convert(varchar(50), SERVERPROPERTY(''Servername''))) = Upper(@TargetServer)
	Begin
		Set @CurrentServer = ''secondary''
		Raiserror(''Cette procédure doit être exécutée sur le serveur primaire...'', 15, 16) with nowait;
		GOTO FinalyseProcess;
	End
	Else
	Begin
		Raiserror(''Cette procédure doit être exécutée sur le serveur primaire...'', 15, 16) with nowait;
		GOTO FinalyseProcess;
	End
	--Select @CurrentServer
	--RETURN;

	/*
	Ici j''utilise les étiquetes pour décider du Workflow en fonction du serveur courant (Source [primary] ou cible [secondary])
	*/
	IsPrimaryServer:
	Begin
		--Suppression et Création du serveur lié
		--Begin
		IF Exists(select top 1 1 from sys.servers where is_linked = 1 and (Upper(name) = UPPER(@LinkedServerDisplayName) Or Upper(data_source) = UPPER(@LinkedServerDisplayName)))
		Begin
			Set @LogMessage = ''Le serveur lié [''+@LinkedServerDisplayName+''] va être supprimé.''

			Raiserror(@LogMessage, 0, 1) with nowait;
			EXEC master.dbo.sp_dropserver @server=@LinkedServerDisplayName--, @droplogins=''droplogins''
		End
		Set @LogMessage = ''Création du serveur lié ''+@LinkedServerDisplayName+'' vers l''''instance [''+@TargetServer+''] pour les Synonymes.''
		Raiserror(@LogMessage, 0, 1) with nowait;
		EXEC master.dbo.sp_addlinkedserver @server = @LinkedServerDisplayName, @srvproduct=N'''', @provider=N''SQLOLEDB'', @datasrc=@TargetServer
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''collation compatible'', @optvalue=N''false''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''data access'', @optvalue=N''true''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''dist'', @optvalue=N''false''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''pub'', @optvalue=N''false''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''rpc'', @optvalue=N''true''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''rpc out'', @optvalue=N''true''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''sub'', @optvalue=N''false''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''connect timeout'', @optvalue=N''0''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''collation name'', @optvalue=null
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''lazy schema validation'', @optvalue=N''false''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''query timeout'', @optvalue=N''0''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''use remote collation'', @optvalue=N''true''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''remote proc transaction promotion'', @optvalue=N''true''
		EXEC master.dbo.sp_addlinkedsrvlogin @rmtsrvname = @LinkedServerDisplayName, @locallogin = NULL , @useself = N''True'', @rmtuser = N''''
		--Pause une seconde
		WaitFor Delay ''00:00:01'';
		--Tests du serveur lié
		Begin try
			Exec sp_testlinkedserver @LinkedServerDisplayName;  
			Set @LogMessage = ''La connexion au serveur lié est validée!''
			Raiserror(@LogMessage, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ''Erreur de connexion au serveur lié [''+@LinkedServerDisplayName+'']: '' + ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,16, 16) with nowait;
			Set @OnErrorStep = ''Linked server connection failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch
		--End

		--Création des synonymes et chargement des tables
		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_InstanceLoginsRoles'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_InstanceLoginsRoles]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_InstanceLoginsRoles] FOR [''+@LinkedServerDisplayName+''].[msdb].[LogShip].[InstanceLoginsRoles];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;

			Delete [LogShip].[syn_InstanceLoginsRoles]

			Insert into [LogShip].[syn_InstanceLoginsRoles]
			Select * from [LogShip].[InstanceLoginsRoles] 
			Set @NbRows = @@ROWCOUNT
			Raiserror(''Table LogShip.InstanceLoginsRoles %d lignes copiée(s)'', 0, 1, @NbRows) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_InstanceLoginsRoles failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch
		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_DbFilesLocation'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_DbFilesLocation]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_DbFilesLocation] FOR [''+@LinkedServerDisplayName+''].[msdb].[LogShip].[DbFilesLocation];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;

			Delete [LogShip].[syn_DbFilesLocation] Where DatabaseName = Case When @LSDatabaseName is null Then DatabaseName Else @LSDatabaseName End;

			Insert into [LogShip].[syn_DbFilesLocation]
			Select * from [LogShip].[DbFilesLocation] 
			Where DatabaseName = Case When @LSDatabaseName is null Then DatabaseName Else @LSDatabaseName End
			--And DatabaseName not in(''tempdb'', ''master'', ''msdb'', ''model'')
			Set @NbRows = @@ROWCOUNT
			Raiserror(''Table LogShip.DbFilesLocation %d lignes copiée(s)'', 0, 1, @NbRows) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_DbFilesLocation failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_DatabasesDetails'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_DatabasesDetails]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_DatabasesDetails] FOR [''+@LinkedServerDisplayName+''].[msdb].[LogShip].[DatabasesDetails];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;

			Delete [LogShip].[syn_DatabasesDetails] Where DatabaseName = Case When @LSDatabaseName is null Then DatabaseName Else @LSDatabaseName End;

			Insert into [LogShip].[syn_DatabasesDetails]([ServerInstanceName], [InstanceName], [DatabaseName], [RecoveryModel], [NewRecoveryModel], [OriginalRecoveryModel], [VersionNum], [NewVersionNum], [CompatibilityLevel])
			Select [ServerInstanceName], [InstanceName], [DatabaseName], [RecoveryModel], [NewRecoveryModel], [OriginalRecoveryModel], [VersionNum], [NewVersionNum], [CompatibilityLevel]
			from [LogShip].[DatabasesDetails]
			Where DatabaseName = Case When @LSDatabaseName is null Then DatabaseName Else @LSDatabaseName End
			Set @NbRows = @@ROWCOUNT
			Raiserror(''Table LogShip.DatabasesDetails %d lignes copiée(s)'', 0, 1, @NbRows) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_DatabasesDetails failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sysschedules'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_DatabasesDetails]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sysschedules] FOR [''+@LinkedServerDisplayName+''].[msdb].[dbo].[sysschedules];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sysschedules failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sysjobs'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_DatabasesDetails]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sysjobs] FOR [''+@LinkedServerDisplayName+''].[msdb].[dbo].[sysjobs];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sysjobs failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch


		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_up_Restore_LogShipping_DB'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_up_Restore_LogShipping_DB]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_up_Restore_LogShipping_DB] FOR [''+@LinkedServerDisplayName+''].[msdb].[LogShip].[up_Restore_LogShipping_DB];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_up_Restore_LogShipping_DB failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		--Ajout du synonymes sur le serveur cible
		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_up_Set_LogShipping_ON'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_up_Set_LogShipping_ON]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_up_Set_LogShipping_ON] FOR [''+@LinkedServerDisplayName+''].[msdb].[LogShip].[up_Set_LogShipping_ON];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_up_Set_LogShipping_ON failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		--Paramétrage du log shipping côté SECONDARY
		--Begin
		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_add_log_shipping_secondary_primary'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_up_Set_LogShipping_ON]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sp_add_log_shipping_secondary_primary] FOR [''+@LinkedServerDisplayName+''].[master].[dbo].[sp_add_log_shipping_secondary_primary];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sp_add_log_shipping_secondary_primary failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_add_log_shipping_secondary_database'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_sp_add_log_shipping_secondary_database]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sp_add_log_shipping_secondary_database] FOR [''+@LinkedServerDisplayName+''].[master].[dbo].[sp_add_log_shipping_secondary_database];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sp_add_log_shipping_secondary_database failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_log_shipping_secondary'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_log_shipping_secondary]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_log_shipping_secondary] FOR [''+@LinkedServerDisplayName+''].[msdb].[dbo].[log_shipping_secondary];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_log_shipping_secondary failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_add_schedule'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_sp_add_schedule]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sp_add_schedule] FOR [''+@LinkedServerDisplayName+''].[msdb].[dbo].[sp_add_schedule];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sp_add_schedule failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_delete_job'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_sp_delete_job]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sp_delete_job] FOR [''+@LinkedServerDisplayName+''].[msdb].[dbo].[sp_delete_job];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sp_delete_job failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_update_job'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_sp_update_job]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sp_update_job] FOR [''+@LinkedServerDisplayName+''].[msdb].[dbo].[sp_update_job];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sp_update_job failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_attach_schedule'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_sp_attach_schedule]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sp_attach_schedule] FOR [''+@LinkedServerDisplayName+''].[msdb].[dbo].[sp_attach_schedule];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sp_attach_schedule failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_delete_log_shipping_secondary_database'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_sp_delete_log_shipping_secondary_database]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sp_delete_log_shipping_secondary_database] FOR [''+@LinkedServerDisplayName+''].[master].[dbo].[sp_delete_log_shipping_secondary_database];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sp_delete_log_shipping_secondary_database failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch
		
		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_start_job'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_sp_start_job]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sp_start_job] FOR [''+@LinkedServerDisplayName+''].[msdb].[dbo].[sp_start_job];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sp_start_job failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch
		--End

		--Boucle sur les BDD pour changer le RECOVERY si besoin et lancer le Backup FULL	
		IF OBJECT_ID(''tempdb..#tmp'') is not null
		Begin
			Drop table #tmp;
		End
		Select Identity(int, 1, 1) as RowId, DatabaseName
		, NewRecoveryModel
		, (''Backup Database ''+QUOTENAME(DatabaseName)+'' To ''+Replace(@BackupToDiskString, ''$DatabaseName$'', DatabaseName)+'';'') as BackupCmd
		, ''Exec msdb.LogShip.syn_up_Restore_LogShipping_DB ''''''+DatabaseName+'''''', ''''''+Replace(Replace(@BackupToDiskString, ''$DatabaseName$'', DatabaseName), '''''''', '''''''''''')+'''''', null, null;'' as RestoreCmd
		into #tmp from [LogShip].[DatabasesDetails]
		Where [DatabaseName] in (SELECT [DatabaseName] From #LSDatabases)

		--Boucle sur les bases de données	
		Set @RowId = 0;
		Set @CreateSynCmd = '''';
		Set @DropSynCmd = '''';
	
			
		While Exists(Select top 1 1 from #tmp Where RowId > @RowId Order by RowId ASC)
		Begin
			--Récup des ID et de la planification côté source
			select @LS_BackUpScheduleID = schedule_id, @LS_BackUpScheduleUID = schedule_uid from dbo.sysschedules
			Where name = ''Logship every 10 minutes''
			Select @LS_BackupJobId = job_id from dbo.sysjobs Where name = ''SoSql_LogShip_Backup_All_DB''

			Select top 1 @RowId = RowId, @CreateSynCmd = (ISNULL(NewRecoveryModel, '''') + CHAR(13)+BackupCmd)
			, @DropSynCmd = RestoreCmd
			, @DatabaseName = DatabaseName
			from #tmp Where RowId > @RowId Order by RowId ASC
		
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
			Exec(@CreateSynCmd)

			Raiserror(@DropSynCmd, 0, 1) with nowait;
			Exec(@DropSynCmd)
			Set @CreateSynCmd = '''';
			Set @DropSynCmd = '''';

			--Autorise la mise en place du log shipping (add try catch bloc)
			Set @EnableLogShipping = 1;

			IF @EnableLogShipping = 1
			Begin

				IF @LS_BackUpScheduleID is not null
				Begin
					Begin try
						Raiserror(''Ajout de la base de données primaire [%s] au log shipping...'', 0, 1, @DatabaseName) with nowait;
						EXEC @SP_Add_RetCode = master.dbo.sp_add_log_shipping_primary_database 
								@database = @DatabaseName 
								,@backup_directory = @BackupShare 
								,@backup_share = @BackupShare 
								,@backup_job_name = N''LS_default_Job'' 
								,@backup_retention_period = @LogFilesRetentionMinutes
								,@backup_compression = 1
								,@backup_threshold = 60 
								,@threshold_alert_enabled = 1
								,@history_retention_period = @LogFilesRetentionMinutes 
								,@overwrite = 1 	

								Update msdb.dbo.log_shipping_primary_databases
								Set backup_job_id = @LS_BackupJobId
								Where primary_database = @DatabaseName

								Raiserror(''Attachement du schedule...'', 0, 1) with nowait;
								EXEC msdb.dbo.sp_attach_schedule 
										@job_id = @LS_BackupJobId 
										,@schedule_id = @LS_BackUpScheduleID  

								EXEC msdb.dbo.sp_update_job 
										@job_id = @LS_BackupJobId 
										,@enabled = 1 

								Raiserror(''Création de l''''alerte...'', 0, 1) with nowait;
								EXEC master.dbo.sp_add_log_shipping_alert_job;

								Raiserror(''Ajout de la base de données secondaire [%s]...'', 0, 1, @DatabaseName) with nowait;
								EXEC master.dbo.sp_add_log_shipping_primary_secondary 
										@primary_database = @DatabaseName 
										,@secondary_server = @TargetServer 
										,@secondary_database = @DatabaseName 
										,@overwrite = 1 
								Raiserror(''Suppression du job temporaire LS_default_Job...'', 0, 1) with nowait;
								Exec msdb.dbo.sp_delete_job @job_name = ''LS_default_Job''

								--Work on target server
								Raiserror(''Paramétrage du Log Shipping sur la cible...'', 0, 1) with nowait;
								--Récup des ID et de la planification côté destination
								select @LS_BackUpScheduleID = schedule_id, @LS_BackUpScheduleUID = schedule_uid from msdb.LogShip.syn_sysschedules
								Where name = ''Retore log every 10 minutes''
								Select @LS_BackupJobId = job_id from msdb.LogShip.syn_sysjobs Where name = ''SoSql_LogShip_Restore_All_DB''
								Select @LS_Secondary_CopyJobId = job_id from msdb.LogShip.syn_sysjobs Where name = ''SoSql_LogShip_Copy_All_DB''

								If OBJECT_ID(''LogShip.syn_sp_add_log_shipping_secondary_primary'') is not null
								Begin
									EXEC LogShip.syn_sp_add_log_shipping_secondary_primary 
											@primary_server = @SourceServer 
											,@primary_database = @DatabaseName 
											,@backup_source_directory = @BackupShare 
											,@backup_destination_directory = @BackupShare 
											,@copy_job_name = N''LS_default_Copy_Job'' 
											,@restore_job_name = N''LS_default_Job'' 
											,@file_retention_period = @LogFilesRetentionMinutes 
											,@overwrite = 1 
								End

								Update LogShip.syn_log_shipping_secondary
								Set restore_job_id = @LS_BackupJobId, copy_job_id = @LS_Secondary_CopyJobId
								Where primary_database = @DatabaseName

								Raiserror(''Attachement du schedule...'', 0, 1) with nowait;
								EXEC LogShip.syn_sp_attach_schedule 
										@job_id = @LS_BackupJobId 
										,@schedule_id = @LS_BackUpScheduleID  


								If OBJECT_ID(''LogShip.syn_sp_add_log_shipping_secondary_database'') is not null
								Begin

									EXEC LogShip.syn_sp_add_log_shipping_secondary_database
											@secondary_database = @DatabaseName
											,@primary_server = @SourceServer 
											,@primary_database = @DatabaseName
											,@restore_delay = 0 
											,@restore_mode = @RestoreMode 
											,@disconnect_users	= 0 
											,@restore_threshold = 45   
											,@threshold_alert_enabled = 1 
											,@history_retention_period	= @LogFilesRetentionMinutes 
											,@overwrite = 1 
									Exec LogShip.syn_sp_delete_job @job_name = ''LS_default_Job''
									Exec LogShip.syn_sp_delete_job @job_name = ''LS_default_Copy_Job''
								End

								EXEC LogShip.syn_sp_update_job 
										@job_id = @LS_BackupJobId 
										,@enabled = 1
								EXEC LogShip.syn_sp_update_job 
										@job_id = @LS_Secondary_CopyJobId 
										,@enabled = 1

					End try
					Begin catch
						SET @LogMessage = ERROR_MESSAGE() ; 
						Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
						Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
						Set @OnErrorStep = ''Erreur de création du log shipping'';
						Set @OnErrorNextStep = ''Rollback all'';
						Raiserror(@OnErrorStep,0,1) with nowait;
						Raiserror(@OnErrorNextStep,0,1) with nowait;
						GOTO RollbackOnError;
					End catch
				End
			End
		End

		GOTO DropLinkedServer
	End

	IsSecondaryServer:
	Begin
		Raiserror(''Serveur secondaire'', 0, 1) with nowait;
		--Récup des ID de la planification
		select @LS_BackUpScheduleID = schedule_id, @LS_BackUpScheduleUID = schedule_uid from dbo.sysschedules
		Where name = ''Logship every 10 minutes''
		Select @LS_BackupJobId = job_id from dbo.sysjobs Where name = ''SoSql_LogShip_Restore_All_DB''
		--Boucle sur les bases de données	
		Set @RowId = 0;
		Set @CreateSynCmd = '''';
		Set @DropSynCmd = '''';


	End

	RollbackOnError:
	Begin
		IF @OnErrorNextStep = ''Drop linked server''
		Begin
			GOTO DropLinkedServer;
		End

		IF @OnErrorNextStep = ''Rollback all''
		Begin
			GOTO RollbackAll;
		End

		Raiserror(''Une erreur c''''est produite'', 0, 1) with nowait;
	End

	RollbackAll:
	Begin
		--Boucle sur les bases de donnéesen log shipping
		Set @CreateSynCmd = '''';
		Set @DropSynCmd = '''';
		Declare @UnknownJobId uniqueidentifier = NewId()

		Update msdb.dbo.log_shipping_primary_databases
		Set backup_job_id = @UnknownJobId

		Update [LogShip].[syn_log_shipping_secondary]
		Set copy_job_id = @UnknownJobId, restore_job_id = @UnknownJobId
			
		While Exists(Select top 1 1 from msdb.dbo.log_shipping_primary_databases where primary_database = Case When @LSDatabaseName is null Then primary_database Else @LSDatabaseName End)
		Begin
			Set @CreateSynCmd = '''';
			Set @DropSynCmd = '''';
			Set @DatabaseName = '''';

			Select  top 1 
			@CreateSynCmd = ''Exec master..sp_delete_log_shipping_primary_secondary ''''''+primary_database+'''''', N''''''+@TargetServer+'''''', ''''''+primary_database+'''''';
			Exec master..sp_delete_log_shipping_primary_database ''''''+primary_database+''''''''
			, @DropSynCmd = ''Exec LogShip.syn_sp_delete_log_shipping_secondary_database ''''''+primary_database+''''''''
			, @DatabaseName = primary_database
			from msdb.dbo.log_shipping_monitor_primary

			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;

			IF Exists(Select Top 1 1 from [LogShip].[syn_log_shipping_secondary] Where primary_database = @DatabaseName)
			Begin
				Exec(@DropSynCmd)
				Raiserror(@DropSynCmd, 0, 1) with nowait;				
			End

			Select  
			 @CreateSynCmd = ISNULL(OriginalRecoveryModel, '''')
			from [LogShip].[DatabasesDetails]
			Where DatabaseName = @DatabaseName

			Raiserror(@CreateSynCmd, 0, 1) with nowait;
			Exec(@CreateSynCmd)
		End
		GOTO DropLinkedServer;
	End

	DropLinkedServer:
	Begin
		--Suppression des synonymes et du serveur lié par sécurité
		Raiserror(''Suppression des synonymes et du serveur lié par sécurité !'', 0, 1) with nowait;

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_DbFilesLocation'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_DbFilesLocation]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_up_Set_LogShipping_ON'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_up_Set_LogShipping_ON]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_DatabasesDetails'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_DatabasesDetails]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_start_job'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_start_job]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sysjobs'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sysjobs]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sysschedules'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sysschedules]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_up_Restore_LogShipping_DB'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_up_Restore_LogShipping_DB]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_add_log_shipping_secondary_primary'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_add_log_shipping_secondary_primary]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_add_log_shipping_secondary_database'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_add_log_shipping_secondary_database]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_log_shipping_secondary'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_log_shipping_secondary]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_add_log_shipping_secondary_database'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_add_log_shipping_secondary_database]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_update_job'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_update_job]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_delete_job'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_delete_job]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_delete_log_shipping_secondary_database'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_delete_log_shipping_secondary_database]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_attach_schedule'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_attach_schedule]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_add_schedule'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_add_schedule]
		End

		IF Exists(select top 1 1 from sys.servers where is_linked = 1 and (Upper(name) = UPPER(@LinkedServerDisplayName) Or Upper(data_source) = UPPER(@LinkedServerDisplayName)))
		Begin
			Set @LogMessage = ''Le serveur lié [''+@LinkedServerDisplayName+''] va être supprimé.''
			Raiserror(@LogMessage, 0, 1) with nowait;
			EXEC master.dbo.sp_dropserver @server=@LinkedServerDisplayName--, @droplogins=''droplogins''
		End
		
		GOTO FinalyseProcess;
	End

	FinalyseProcess:
END
'
End

IF (SELECT OBJECT_ID(N'[LogShip].[up_Set_LogShipping_OFF]')) is null
Begin
execute dbo.sp_executesql @statement = N'
/* =============================================
-- Author:		JFP SoSql
-- Create date: 2021-15-11
-- Description:	Désactive le log shipping
-- Select * from [LogShip].[DatabasesDetails]
-- Delete [LogShip].[DatabasesDetails] Where DatabaseName = ''DemoDB''

--Désactive le Log Shipping sur toutes les bases de données
Exec LogShip.up_Set_LogShipping_OFF

--Désactive le Log Shipping sur une base de données
Exec msdb.LogShip.up_Set_LogShipping_OFF @LSDatabaseName = ''DemoDB10''
 =============================================*/
CREATE PROCEDURE [LogShip].[up_Set_LogShipping_OFF]
(
@SetSourceDbOffLine bit = 0,
@LSDatabaseName sysname = null
)
AS
BEGIN
	SET NOCOUNT ON;
	IF OBJECT_ID(''tempdb..#LSDatabases'') is not null
	Begin
		Drop table #LSDatabases;
	End

	--Retourne la liste des bases du Log Shipping
	SELECT [DatabaseName] into #LSDatabases FROM [msdb].[LogShip].[DatabasesDetails]
	Where [DatabaseName] = Case When @LSDatabaseName is null Then [DatabaseName] Else @LSDatabaseName End

	Delete #LSDatabases
	Where [DatabaseName] not in(SELECT primary_database FROM msdb.dbo.log_shipping_primary_databases)
	--Variables du WorkFlow
	--Begin
	Declare @LogMessage VarChar(MAX), @DateString varchar(50)
	Declare @RowId int = 0, @SynName nvarchar(128), @SynFullName nvarchar(128), @CreateSynCmd nvarchar(4000), @DropSynCmd nvarchar(4000);
	Declare @IsNullable bit = 1;
	Declare @ErrorSeverity int, @ErrorState int;
	Declare @LinkedServerDisplayName Sysname = ''LogShipLinkedServer'';
	Declare @NbRows int
	Declare @BackupDirectory varchar(255), @BackupShare varchar(255), @SourceServer varchar(255), @TargetServer varchar(255), @TargetCompatibilityLevel varchar(50);
	Declare @EnableLogShipping bit = 0, @DatabaseName sysname;
	Declare @LS_BackupJobId	AS uniqueidentifier 
	Declare @LS_PrimaryId	AS uniqueidentifier 
	Declare @LS_AlertJobId	AS uniqueidentifier 
	Declare @SP_Add_RetCode	As int 
	Declare @LS_BackUpScheduleUID	As uniqueidentifier 
	Declare @LS_BackUpScheduleID	AS int
	Declare @LS_TargetBackUpScheduleUID	As uniqueidentifier 
	Declare @LS_TargetBackUpScheduleID	AS int
	Declare @LS_TargetBackupJobId	AS uniqueidentifier 
	Declare @LS_Secondary_CopyJobId	AS uniqueidentifier 
	Declare @OnErrorStep varchar(50)
	Declare @OnErrorNextStep varchar(50)
	Declare @LogFilesRetentionMinutes int
	--End
	IF not Exists(Select top 1 1 from #LSDatabases)
	Begin
		IF @LSDatabaseName is null
		Begin
			Raiserror(''Il n''''y a aucune base de données associée à ce script en LOG SHIPPING !'', 0, 1) with nowait;
		End
		ELSE
		Begin
			Raiserror(''La base de données [%s] n''''est pas en LOG SHIPPING !'', 0, 1, @LSDatabaseName) with nowait;
		End
		GOTO FinalyseProcess;
	End	
	Select @BackupShare = ParamValue from LogShip.LogShipParams Where ParamName = ''BackupShare''
	Select @SourceServer = ParamValue from LogShip.LogShipParams Where ParamName = ''SourceServer''
	Select @TargetServer = ParamValue from LogShip.LogShipParams Where ParamName = ''TargetServer''
	Select @LogFilesRetentionMinutes = Convert(int, ParamValue) from LogShip.LogShipParams Where ParamName = ''LogFilesRetentionMinutes''

	--Vérifie sur quel serveur la procedure est exécutée
	Declare @CurrentServer varchar(50)
	IF Upper(Convert(varchar(50), SERVERPROPERTY(''machinename''))) = Upper(@SourceServer) Or Upper(Convert(varchar(50), SERVERPROPERTY(''Servername''))) = Upper(@SourceServer)--, SERVERPROPERTY(''instancename'')
	Begin
		Set @CurrentServer = ''primary''
		Raiserror(''Exécution sur le serveur primaire...'', 0, 1) with nowait;
		GOTO IsPrimaryServer
	End
	Else IF Upper(Convert(varchar(50), SERVERPROPERTY(''machinename''))) = Upper(@TargetServer) Or Upper(Convert(varchar(50), SERVERPROPERTY(''Servername''))) = Upper(@TargetServer)
	Begin
		Set @CurrentServer = ''secondary''
		Raiserror(''Cette procédure doit être exécutée sur le serveur primaire...'', 15, 16) with nowait;
		GOTO FinalyseProcess;
	End
	Else
	Begin
		Raiserror(''Cette procédure doit être exécutée sur le serveur primaire...'', 15, 16) with nowait;
		GOTO FinalyseProcess;
	End
	--Select @CurrentServer
	--RETURN;

	/*
	Ici j''utilise les étiquetes pour décider du Workflow en fonction du serveur courant (Source [primary] ou cible [secondary])
	*/
	IsPrimaryServer:
	Begin
		--Suppression et Création du serveur lié
		--Begin
		IF Exists(select top 1 1 from sys.servers where is_linked = 1 and (Upper(name) = UPPER(@LinkedServerDisplayName) Or Upper(data_source) = UPPER(@LinkedServerDisplayName)))
		Begin
			Set @LogMessage = ''Le serveur lié [''+@LinkedServerDisplayName+''] va être supprimé.''

			Raiserror(@LogMessage, 0, 1) with nowait;
			EXEC master.dbo.sp_dropserver @server=@LinkedServerDisplayName--, @droplogins=''droplogins''
		End
		Set @LogMessage = ''Création du serveur lié ''+@LinkedServerDisplayName+'' vers l''''instance [''+@TargetServer+''] pour les Synonymes.''
		Raiserror(@LogMessage, 0, 1) with nowait;
		EXEC master.dbo.sp_addlinkedserver @server = @LinkedServerDisplayName, @srvproduct=N'''', @provider=N''SQLOLEDB'', @datasrc=@TargetServer
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''collation compatible'', @optvalue=N''false''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''data access'', @optvalue=N''true''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''dist'', @optvalue=N''false''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''pub'', @optvalue=N''false''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''rpc'', @optvalue=N''true''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''rpc out'', @optvalue=N''true''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''sub'', @optvalue=N''false''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''connect timeout'', @optvalue=N''0''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''collation name'', @optvalue=null
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''lazy schema validation'', @optvalue=N''false''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''query timeout'', @optvalue=N''0''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''use remote collation'', @optvalue=N''true''
		EXEC master.dbo.sp_serveroption @server=@LinkedServerDisplayName, @optname=N''remote proc transaction promotion'', @optvalue=N''true''
		EXEC master.dbo.sp_addlinkedsrvlogin @rmtsrvname = @LinkedServerDisplayName, @locallogin = NULL , @useself = N''True'', @rmtuser = N''''
		--Pause une seconde
		WaitFor Delay ''00:00:01'';
		--Tests du serveur lié
		Begin try
			Exec sp_testlinkedserver @LinkedServerDisplayName;  
			Set @LogMessage = ''La connexion au serveur lié est validée!''
			Raiserror(@LogMessage, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ''Erreur de connexion au serveur lié [''+@LinkedServerDisplayName+'']: '' + ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,16, 16) with nowait;
			Set @OnErrorStep = ''Linked server connection failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch
		--End

		--Création des synonymes et chargement des tables
		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_DbFilesLocation'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_DbFilesLocation]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_DbFilesLocation] FOR [''+@LinkedServerDisplayName+''].[msdb].[LogShip].[DbFilesLocation];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;

			Delete [LogShip].[syn_DbFilesLocation];

			Insert into [LogShip].[syn_DbFilesLocation]
			Select * from [LogShip].[DbFilesLocation] Where DatabaseName not in(''tempdb'', ''master'', ''msdb'', ''model'')
			Set @NbRows = @@ROWCOUNT
			Raiserror(''Table LogShip.DbFilesLocation %d lignes copiée(s)'', 0, 1, @NbRows) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_DbFilesLocation failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_DatabasesDetails'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_DatabasesDetails]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_DatabasesDetails] FOR [''+@LinkedServerDisplayName+''].[msdb].[LogShip].[DatabasesDetails];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;

			Delete [LogShip].[syn_DatabasesDetails];

			Insert into [LogShip].[syn_DatabasesDetails]([ServerInstanceName], [InstanceName], [DatabaseName], [RecoveryModel], [NewRecoveryModel], [OriginalRecoveryModel], [VersionNum], [NewVersionNum], [CompatibilityLevel])
			Select [ServerInstanceName], [InstanceName], [DatabaseName], [RecoveryModel], [NewRecoveryModel], [OriginalRecoveryModel], [VersionNum], [NewVersionNum], [CompatibilityLevel]
			from [LogShip].[DatabasesDetails]
			Set @NbRows = @@ROWCOUNT
			Raiserror(''Table LogShip.DatabasesDetails %d lignes copiée(s)'', 0, 1, @NbRows) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_DatabasesDetails failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sysschedules'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_DatabasesDetails]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sysschedules] FOR [''+@LinkedServerDisplayName+''].[msdb].[dbo].[sysschedules];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sysschedules failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sysjobs'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_DatabasesDetails]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sysjobs] FOR [''+@LinkedServerDisplayName+''].[msdb].[dbo].[sysjobs];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sysjobs failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch


		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_up_Restore_LogShipping_DB'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_up_Restore_LogShipping_DB]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_up_Restore_LogShipping_DB] FOR [''+@LinkedServerDisplayName+''].[msdb].[LogShip].[up_Restore_LogShipping_DB];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_up_Restore_LogShipping_DB failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		--Ajout du synonymes sur le serveur cible
		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_up_Set_LogShipping_ON'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_up_Set_LogShipping_ON]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_up_Set_LogShipping_ON] FOR [''+@LinkedServerDisplayName+''].[msdb].[LogShip].[up_Set_LogShipping_ON];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_up_Set_LogShipping_ON failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		--Paramétrage du log shipping côté SECONDARY
		--Begin
		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_add_log_shipping_secondary_primary'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_up_Set_LogShipping_ON]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sp_add_log_shipping_secondary_primary] FOR [''+@LinkedServerDisplayName+''].[master].[dbo].[sp_add_log_shipping_secondary_primary];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sp_add_log_shipping_secondary_primary failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_add_log_shipping_secondary_database'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_sp_add_log_shipping_secondary_database]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sp_add_log_shipping_secondary_database] FOR [''+@LinkedServerDisplayName+''].[master].[dbo].[sp_add_log_shipping_secondary_database];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sp_add_log_shipping_secondary_database failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_log_shipping_secondary'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_log_shipping_secondary]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_log_shipping_secondary] FOR [''+@LinkedServerDisplayName+''].[msdb].[dbo].[log_shipping_secondary];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_log_shipping_secondary failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_add_schedule'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_sp_add_schedule]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sp_add_schedule] FOR [''+@LinkedServerDisplayName+''].[msdb].[dbo].[sp_add_schedule];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sp_add_schedule failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_delete_job'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_sp_delete_job]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sp_delete_job] FOR [''+@LinkedServerDisplayName+''].[msdb].[dbo].[sp_delete_job];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sp_delete_job failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_update_job'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_sp_update_job]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sp_update_job] FOR [''+@LinkedServerDisplayName+''].[msdb].[dbo].[sp_update_job];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sp_update_job failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_attach_schedule'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_sp_attach_schedule]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sp_attach_schedule] FOR [''+@LinkedServerDisplayName+''].[msdb].[dbo].[sp_attach_schedule];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sp_attach_schedule failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch

		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_delete_log_shipping_secondary_database'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_sp_delete_log_shipping_secondary_database]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sp_delete_log_shipping_secondary_database] FOR [''+@LinkedServerDisplayName+''].[master].[dbo].[sp_delete_log_shipping_secondary_database];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sp_delete_log_shipping_secondary_database failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch
		
		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_start_job'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_sp_start_job]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sp_start_job] FOR [''+@LinkedServerDisplayName+''].[msdb].[dbo].[sp_start_job];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sp_start_job failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch
		
		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sf_IsRunningJob'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_sf_IsRunningJob]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_sf_IsRunningJob] FOR [''+@LinkedServerDisplayName+''].[msdb].[LogShip].[sf_IsRunningJob];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_sf_IsRunningJob failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch
		
		Begin try
			IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_up_WaitRunningJob'' AND schema_id = SCHEMA_ID(N''LogShip''))
			Begin
				DROP SYNONYM [LogShip].[syn_up_WaitRunningJob]
			End
			Set @CreateSynCmd = ''CREATE SYNONYM [LogShip].[syn_up_WaitRunningJob] FOR [''+@LinkedServerDisplayName+''].[msdb].[LogShip].[up_WaitRunningJob];'';
			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;
		End try
		Begin catch
			SET @LogMessage = ERROR_MESSAGE() ; 
			Select @ErrorSeverity = ERROR_SEVERITY(), @ErrorState = ERROR_STATE();
			Raiserror(@LogMessage,@ErrorSeverity,@ErrorState) with nowait;
			Set @OnErrorStep = ''Create syn_up_WaitRunningJob failed'';
			Set @OnErrorNextStep = ''Drop linked server'';
			GOTO RollbackOnError;
		End catch
		--End 

		--Boucle sur les bases de donnéesen en log shipping
		Set @CreateSynCmd = '''';
		Set @DropSynCmd = '''';
		Declare @UnknownJobId uniqueidentifier = NewId()

		--Fait un dernier backup et un dernier restore
		EXEC msdb.dbo.sp_update_job @job_name = ''SoSql_LogShip_Backup_All_DB'',@enabled = 0
		Raiserror(''Désactive le job de Backup du Log Shipping et l''''exécute une dernière fois'', 0, 1) with nowait;
		If (select LogShip.sf_IsRunningJob(''SoSql_LogShip_Backup_All_DB'')) = 0
		Begin
			EXEC msdb.dbo.sp_start_job N''SoSql_LogShip_Backup_All_DB'';
			Waitfor delay ''00:00:05''
		End

		While (select LogShip.sf_IsRunningJob(''SoSql_LogShip_Backup_All_DB'')) = 1
		Begin
			Waitfor delay ''00:00:05''
		End

		EXEC LogShip.syn_sp_update_job @job_name = ''SoSql_LogShip_Restore_All_DB'',@enabled = 0
		EXEC LogShip.syn_sp_update_job @job_name = ''SoSql_LogShip_Copy_All_DB'',@enabled = 0

		EXEC [LogShip].[syn_up_WaitRunningJob] N''SoSql_LogShip_Restore_All_DB'', ''00:00:05'';
		EXEC LogShip.syn_sp_start_job N''SoSql_LogShip_Restore_All_DB'';
		Waitfor delay ''00:00:05''

		EXEC [LogShip].[syn_up_WaitRunningJob] N''SoSql_LogShip_Restore_All_DB'', ''00:00:05'';

		Update msdb.dbo.log_shipping_primary_databases
		Set backup_job_id = @UnknownJobId
		Where primary_database in(SELECT [DatabaseName] FROM #LSDatabases)

		Update [LogShip].[syn_log_shipping_secondary]
		Set copy_job_id = @UnknownJobId, restore_job_id = @UnknownJobId
		Where Convert(nvarchar(128), primary_database collate French_CI_AS) in(SELECT [DatabaseName] collate French_CI_AS FROM #LSDatabases)

		While Exists(Select top 1 1 from msdb.dbo.log_shipping_primary_databases Where primary_database in(SELECT [DatabaseName] FROM #LSDatabases))
		Begin
			Set @CreateSynCmd = '''';
			Set @DropSynCmd = '''';
			Set @DatabaseName = '''';

			Select  top 1 
			@CreateSynCmd = ''Exec master..sp_delete_log_shipping_primary_secondary ''''''+primary_database+'''''', N''''''+@TargetServer+'''''', ''''''+primary_database+'''''';
			Exec master..sp_delete_log_shipping_primary_database ''''''+primary_database+''''''''
			, @DropSynCmd = ''Exec LogShip.syn_sp_delete_log_shipping_secondary_database ''''''+primary_database+''''''''
			, @DatabaseName = primary_database
			from msdb.dbo.log_shipping_monitor_primary
			Where primary_database in(SELECT [DatabaseName] FROM #LSDatabases)

			Exec(@CreateSynCmd)
			Raiserror(@CreateSynCmd, 0, 1) with nowait;

			IF Exists(Select Top 1 1 from [LogShip].[syn_log_shipping_secondary] Where primary_database = @DatabaseName)
			Begin
				Exec(@DropSynCmd)
				Raiserror(@DropSynCmd, 0, 1) with nowait;				
			End
			--Met la base OFFLINE sur la source
			IF @SetSourceDbOffLine = 1
			Begin
				Exec(''Use master; ALTER DATABASE [''+@DatabaseName+''] SET  SINGLE_USER With Rollback immediate; ALTER DATABASE [''+@DatabaseName+''] SET  OFFLINE;'');
			End
			--Met la base online sur la cible
			Select  
			 @CreateSynCmd = ISNULL(OriginalRecoveryModel, '''')
			 ,@DropSynCmd = ISNULL(NewCompatibilityLevel, '''')
			from [LogShip].[DatabasesDetails]
			Where DatabaseName = @DatabaseName

			Raiserror(''Affecte le compatibility level et le recovery model de la base de données [%s]'', 0, 1, @DatabaseName) with nowait;
			Exec msdb.LogShip.syn_up_Restore_LogShipping_DB @DatabaseName, null, @DropSynCmd, @CreateSynCmd;
			Exec(@CreateSynCmd)
		End

		IF Exists(Select top 1 1 from msdb.dbo.log_shipping_primary_databases Where primary_database in(SELECT [DatabaseName] FROM [msdb].[LogShip].[DatabasesDetails]))
		Begin
			--Réactive les jobs s''il restre des BDD en Log Shipping
			EXEC msdb.dbo.sp_update_job @job_name = ''SoSql_LogShip_Backup_All_DB'',@enabled = 1
			EXEC LogShip.syn_sp_update_job @job_name = ''SoSql_LogShip_Restore_All_DB'',@enabled = 1
			EXEC LogShip.syn_sp_update_job @job_name = ''SoSql_LogShip_Copy_All_DB'',@enabled = 1
		End

		GOTO DropLinkedServer
	End

	RollbackOnError:
	Begin
		IF @OnErrorNextStep = ''Drop linked server''
		Begin
			GOTO DropLinkedServer;
		End
	End

	DropLinkedServer:
	Begin
		--Suppression des synonymes et du serveur lié par sécurité
		Raiserror(''Suppression des synonymes et du serveur lié par sécurité !'', 0, 1) with nowait;

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_DbFilesLocation'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_DbFilesLocation]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_up_Set_LogShipping_ON'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_up_Set_LogShipping_ON]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_up_WaitRunningJob'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_up_WaitRunningJob]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_DatabasesDetails'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_DatabasesDetails]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sysjobs'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sysjobs]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sysschedules'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sysschedules]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_start_job'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_start_job]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sf_IsRunningJob'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sf_IsRunningJob]
		End
		
		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_up_Restore_LogShipping_DB'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_up_Restore_LogShipping_DB]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_add_log_shipping_secondary_primary'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_add_log_shipping_secondary_primary]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_add_log_shipping_secondary_database'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_add_log_shipping_secondary_database]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_log_shipping_secondary'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_log_shipping_secondary]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_add_log_shipping_secondary_database'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_add_log_shipping_secondary_database]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_update_job'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_update_job]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_delete_job'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_delete_job]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_delete_log_shipping_secondary_database'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_delete_log_shipping_secondary_database]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_attach_schedule'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_attach_schedule]
		End

		IF  EXISTS (SELECT * FROM sys.synonyms WHERE name = N''syn_sp_add_schedule'' AND schema_id = SCHEMA_ID(N''LogShip''))
		Begin
			DROP SYNONYM [LogShip].[syn_sp_add_schedule]
		End

		IF Exists(select top 1 1 from sys.servers where is_linked = 1 and (Upper(name) = UPPER(@LinkedServerDisplayName) Or Upper(data_source) = UPPER(@LinkedServerDisplayName)))
		Begin
			Set @LogMessage = ''Le serveur lié [''+@LinkedServerDisplayName+''] va être supprimé.''
			Raiserror(@LogMessage, 0, 1) with nowait;
			EXEC master.dbo.sp_dropserver @server=@LinkedServerDisplayName--, @droplogins=''droplogins''
		End
		
		GOTO FinalyseProcess;
	End

	FinalyseProcess:
END
'
End

IF (SELECT OBJECT_ID(N'[LogShip].[up_WaitRunningJob]')) is null
Begin
execute dbo.sp_executesql @statement = N'
-- =============================================
-- Author:		JFP SoSql
-- Create date: 2021-15-11
-- Description:	Attend la fin d''un job
-- Ex: Exec LogShip.up_WaitRunningJob ''name'', ''00:00:05''
-- =============================================
CREATE PROCEDURE [LogShip].[up_WaitRunningJob](@JobName varchar(255), @Delay char(8))
as
Begin
	While (select LogShip.sf_IsRunningJob(@JobName)) = 1
	Begin
		Waitfor delay @Delay
	End
End
'
End

/*===================================================================================================
Ce script permet de générer un script SQL pour transférer les comptes et les roles.
===================================================================================================*/

Declare @RowId int, @sqlcmd nvarchar(max)


IF object_id('LogShip.InstanceLoginsRoles') is null
Begin
CREATE TABLE LogShip.InstanceLoginsRoles
       (
        RowId int IDENTITY(1, 1)
		,LoginName nvarchar(max)
       ,CreateLoginCmd nvarchar(max)
	   ,CmdType varchar(50)
       )
End

IF object_id('tempdb..#Databases') is not null
Begin
	Drop table #Databases;
End
CREATE TABLE #Databases(RowId int IDENTITY(1, 1), DBName nvarchar(100));

IF object_id('tempdb..#DatabaseRoles') is not null
Begin
	Drop table #DatabaseRoles;
End
CREATE TABLE #DatabaseRoles(
	RowId int IDENTITY(1, 1),
	DbName nvarchar(128) NULL,
	RoleName sysname NOT NULL,
	LoginName sysname NOT NULL,
	DefaultSchema sysname NULL
)

----Create logins scripts
----Begin
--INSERT INTO LogShip.InstanceLoginsRoles (LoginName, CreateLoginCmd,CmdType)
--        SELECT name,
--                'IF NOT EXISTS (SELECT * FROM sys.server_principals WHERE name = N'''
--                + name + ''')
--Begin
--' + '	CREATE LOGIN ' + QUOTENAME(name) + ' WITH PASSWORD='
--                + sys.fn_varbintohexstr(password_hash) + ' HASHED, SID='
--                + sys.fn_varbintohexstr(sid) + ', ' + 'DEFAULT_DATABASE='
--                + QUOTENAME(COALESCE(default_database_name , 'master'))
--                + ', DEFAULT_LANGUAGE='
--                + QUOTENAME(COALESCE(default_language_name , 'us_english'))
--                + ', CHECK_EXPIRATION=' + CASE is_expiration_checked
--                                            WHEN 1 THEN 'ON'
--                                            ELSE 'OFF'
--                                          END + ', CHECK_POLICY='
--                + CASE is_policy_checked
--                    WHEN 1 THEN 'ON'
--                    ELSE 'OFF'
--                  END + '
--End
--Go

--'
--, 'Create Login'
--            FROM
--                sys.sql_logins
--            WHERE
--               (name <> 'sa' and name not like '##MS%')

--INSERT INTO LogShip.InstanceLoginsRoles (LoginName, CreateLoginCmd,CmdType)
--        SELECT name,
--                'IF NOT EXISTS (SELECT * FROM sys.server_principals WHERE name = N'''
--                + name + ''')
--Begin' + '
--	CREATE LOGIN ' + QUOTENAME(name) + ' FROM WINDOWS WITH '
--                + 'DEFAULT_DATABASE='
--                + QUOTENAME(COALESCE(default_database_name , 'master'))
--                + ', DEFAULT_LANGUAGE='
--                + QUOTENAME(COALESCE(default_language_name , 'us_english'))
--                + ';
--End
--Go

--'
--, 'Create Login'
--FROM
--    sys.server_principals
--WHERE
--    (
--	type IN ( 'U' , 'G' )
--    AND name Not In ( 'BUILTIN\Administrators' , 'NT AUTHORITY\SYSTEM' )
--	And name not like 'NT %'
--	And name not like '%NT\%'
--	)
----End

----Create server roles script
----Begin
--INSERT INTO LogShip.InstanceLoginsRoles (LoginName, CreateLoginCmd,CmdType)
--        SELECT L.name,
--                'IF Not Exists(SELECT top 1 1 FROM sys.server_principals L JOIN sys.server_role_members RM  ON  L.principal_id = RM.member_principal_id JOIN sys.server_principals R ON  RM.role_principal_id = R.principal_id WHERE L.name ='''+L.name+''' and R.name = '''+R.name+''')
--Begin	
--	EXEC sp_addsrvrolemember ' + QUOTENAME(L.name) + ', '
--                + QUOTENAME(R.name) + ';
--End
--'
--,'Server Role'
--            FROM
--                sys.server_principals L
--            JOIN sys.server_role_members RM
--            ON  L.principal_id = RM.member_principal_id
--            JOIN sys.server_principals R
--            ON  RM.role_principal_id = R.principal_id
--            WHERE
--                L.type IN ( 'U' , 'G' , 'S' )				
--                AND 
--				(
--					L.name IN (Select LoginName from LogShip.InstanceLoginsRoles where CmdType = 'Create Login')
--					AND L.name not like 'NT S%'
--					AND L.name not like '% NT\%'
--				)
----End

IF OBJECT_ID('LogShip.v_ObjectsScripts') is not null
Begin
	Drop view LogShip.v_ObjectsScripts;
End
execute dbo.sp_executesql @statement = N'CREATE view [LogShip].[v_ObjectsScripts]
as
Select (''Use msdb;''+nchar(13)) as SqlText, -2 as OrderIndex
Union all
Select ''IF Not Exists(Select top 1 1 from sys.schemas Where name = ''''LogShip'''')
Begin
	Exec(''''CREATE SCHEMA [LogShip] AUTHORIZATION [dbo];'''')
End'' as SqlText, -1 as OrderIndex
Union all
Select DefinitionText as SqlText, OrderIndex  from LogShip.LogShipObjects Where OrderIndex = 0
Union all
Select DefinitionText as SqlText, OrderIndex  from LogShip.LogShipObjects Where OrderIndex = 1
Union all
Select CreateLoginCmd as SqlText, 0 as OrderIndex From LogShip.InstanceLoginsRoles
Union all
Select LogShip.sf_ObjectDefinition(''LogShip.sf_ObjectDefinition'') as SqlText, 1 as OrderIndex
Union all
Select LogShip.sf_ObjectDefinition(''LogShip.sf_IsRunningJob'') as SqlText, 1 as OrderIndex
Union all
Select LogShip.sf_ObjectDefinition(''LogShip.sf_hexadecimal'') as SqlText, 2 as OrderIndex
Union all
Select LogShip.sf_ObjectDefinition(''dbo.STRING_SPLIT'') as SqlText, 3 as OrderIndex
Union all
Select LogShip.sf_ObjectDefinition(''LogShip.up_Restore_LogShipping_DB'') as SqlText, 3 as OrderIndex
Union all
Select LogShip.sf_ObjectDefinition(''LogShip.up_WaitRunningJob'') as SqlText, 3 as OrderIndex
Union all
Select LogShip.sf_ObjectDefinition(''LogShip.up_Set_LogShipping_ON'') as SqlText, 4 as OrderIndex
Union all
Select LogShip.sf_ObjectDefinition(''LogShip.up_Set_LogShipping_OFF'') as SqlText, 4 as OrderIndex
Union all
Select DefinitionText as SqlText, 6 as OrderIndex  from LogShip.LogShipObjects Where OrderIndex = 5
Union all
Select DefinitionText as SqlText, 7 as OrderIndex  from LogShip.LogShipObjects Where OrderIndex = 6
'


Set @DefinitionText = N'
IF Exists(SELECT job_id FROM msdb.dbo.sysjobs WHERE name = ''SoSql_LogShip_Backup_All_DB'')
Begin
	EXEC msdb.dbo.sp_delete_job @job_name=N''SoSql_LogShip_Backup_All_DB'', @delete_unused_schedule=1
End

BEGIN TRANSACTION
DECLARE @ReturnCode INT
SELECT @ReturnCode = 0

IF NOT EXISTS (SELECT name FROM msdb.dbo.syscategories WHERE name=N''Log Shipping'' AND category_class=1)
BEGIN
EXEC @ReturnCode = msdb.dbo.sp_add_category @class=N''JOB'', @type=N''LOCAL'', @name=N''Log Shipping''
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

END

DECLARE @jobId BINARY(16)
EXEC @ReturnCode =  msdb.dbo.sp_add_job @job_name=N''SoSql_LogShip_Backup_All_DB'', 
		@enabled=0, 
		@notify_level_eventlog=0, 
		@notify_level_email=0, 
		@notify_level_netsend=0, 
		@notify_level_page=0, 
		@delete_level=0, 
		@description=N''Pilote le log shipping de toutes les bases de données'', 
		@category_name=N''Log Shipping'', 
		@owner_login_name=N''sa'', @job_id = @jobId OUTPUT
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N''LogShip'', 
		@step_id=1, 
		@cmdexec_success_code=0, 
		@on_success_action=1, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N''TSQL'', 
		@command=N''Declare @PrimaryServer varchar(255), @ID uniqueidentifier;
Select @PrimaryServer = ParamValue from [LogShip].[LogShipParams] Where ParamName = ''''SourceServer''''
Declare @xp_cmdshellEnabled int;

DECLARE cursor_log CURSOR FAST_FORWARD FOR
	SELECT primary_id FROM msdb.dbo.log_shipping_primary_databases;
OPEN cursor_log;
FETCH NEXT FROM cursor_log INTO @ID;
WHILE @@FETCH_STATUS = 0
BEGIN
	DECLARE @CMD varchar(4000)
	SET @CMD = ''''"$LOGSHIP_EXECUTABLE_PATH$" -Backup '''' + CAST(@ID AS varchar(50)) + '''' -server ''''+@PrimaryServer

	SELECT @xp_cmdshellEnabled = CONVERT(INT, ISNULL(value, value_in_use))
	FROM  sys.configurations
	WHERE  name = ''''xp_cmdshell'''' ;

	IF @xp_cmdshellEnabled = 0
	Begin
		EXEC sp_configure ''''show advanced options'''', 1;
		RECONFIGURE;
		EXEC sp_configure ''''xp_cmdshell'''', 1;
		RECONFIGURE;
	End
	
	EXEC xp_cmdshell @CMD

	--Désactive CMDSHELL
	IF @xp_cmdshellEnabled = 0
	Begin
		EXEC sp_configure ''''show advanced options'''', 1;
		RECONFIGURE;
		EXEC sp_configure ''''xp_cmdshell'''', @xp_cmdshellEnabled;
		RECONFIGURE;
	End

	FETCH NEXT FROM cursor_log INTO @ID;
END
DEALLOCATE cursor_log'', 
		@database_name=N''msdb'', 
		@flags=0
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_update_job @job_id = @jobId, @start_step_id = 1
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobschedule @job_id=@jobId, @name=N''Logship every 10 minutes'', 
		@enabled=1, 
		@freq_type=4, 
		@freq_interval=1, 
		@freq_subday_type=4, 
		@freq_subday_interval=10, 
		@freq_relative_interval=0, 
		@freq_recurrence_factor=0, 
		@active_start_date=20211115, 
		@active_end_date=99991231, 
		@active_start_time=0, 
		@active_end_time=235959, 
		@schedule_uid=N''88137986-2ffb-4486-b380-e8b3a78ed779''
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobserver @job_id = @jobId, @server_name = N''(local)''
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
COMMIT TRANSACTION
GOTO EndSave
QuitWithRollback:
    IF (@@TRANCOUNT > 0) ROLLBACK TRANSACTION
EndSave:
'

Select @DefinitionText = REPLACE(@DefinitionText, '$LOGSHIP_EXECUTABLE_PATH$', Ltrim(Rtrim(ParamValue)))  
From msdb.LogShip.LogShipParams
Where ParamName = 'sqllogship.exe'

execute dbo.sp_executesql @statement = @DefinitionText;

Set @DefinitionText = N'
Declare @DefinitionText nvarchar(max);
Set @DefinitionText = N''
IF Exists(SELECT job_id FROM msdb.dbo.sysjobs WHERE name = ''''SoSql_LogShip_Restore_All_DB'''')
Begin
	EXEC msdb.dbo.sp_delete_job @job_name=N''''SoSql_LogShip_Restore_All_DB'''', @delete_unused_schedule=1
End

BEGIN TRANSACTION
DECLARE @ReturnCode INT
SELECT @ReturnCode = 0

IF NOT EXISTS (SELECT name FROM msdb.dbo.syscategories WHERE name=N''''Log Shipping'''' AND category_class=1)
BEGIN
EXEC @ReturnCode = msdb.dbo.sp_add_category @class=N''''JOB'''', @type=N''''LOCAL'''', @name=N''''Log Shipping''''
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

END

DECLARE @jobId BINARY(16)
EXEC @ReturnCode =  msdb.dbo.sp_add_job @job_name=N''''SoSql_LogShip_Restore_All_DB'''', 
		@enabled=0, 
		@notify_level_eventlog=0, 
		@notify_level_email=0, 
		@notify_level_netsend=0, 
		@notify_level_page=0, 
		@delete_level=0, 
		@description=N''''Logship all DB.'''', 
		@category_name=N''''Log Shipping'''', 
		@owner_login_name=N''''sa'''', @job_id = @jobId OUTPUT
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N''''Restore logs'''', 
		@step_id=1, 
		@cmdexec_success_code=0, 
		@on_success_action=1, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N''''TSQL'''', 
		@command=N''''DECLARE @ID uniqueidentifier;
Declare @SecondaryServer varchar(255);
Select @SecondaryServer = ParamValue from msdb.LogShip.LogShipParams Where ParamName = ''''''''TargetServer''''''''

Declare @xp_cmdshellEnabled int;

DECLARE cursor_log CURSOR FAST_FORWARD FOR
	SELECT secondary_id FROM msdb.dbo.log_shipping_secondary_databases;
OPEN cursor_log;
FETCH NEXT FROM cursor_log INTO @ID;
WHILE @@FETCH_STATUS = 0
BEGIN
	DECLARE @CMD varchar(4000)
	SET @CMD = ''''''''"$LOGSHIP_EXECUTABLE_PATH$" -Restore '''''''' + CAST(@ID AS varchar(50)) + '''''''' -server ''''''''+@SecondaryServer

	SELECT @xp_cmdshellEnabled = CONVERT(INT, ISNULL(value, value_in_use))
	FROM  sys.configurations
	WHERE  name = ''''''''xp_cmdshell'''''''' ;

	IF @xp_cmdshellEnabled = 0
	Begin
		EXEC sp_configure ''''''''show advanced options'''''''', 1;
		RECONFIGURE;
		EXEC sp_configure ''''''''xp_cmdshell'''''''', 1;
		RECONFIGURE;
	End
	
	EXEC xp_cmdshell @CMD

	--Désactive CMDSHELL
	IF @xp_cmdshellEnabled = 0
	Begin
		EXEC sp_configure ''''''''show advanced options'''''''', 1;
		RECONFIGURE;
		EXEC sp_configure ''''''''xp_cmdshell'''''''', @xp_cmdshellEnabled;
		RECONFIGURE;
	End
	FETCH NEXT FROM cursor_log INTO @ID;
END
DEALLOCATE cursor_log
'''', 
		@database_name=N''''msdb'''', 
		@flags=0
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_update_job @job_id = @jobId, @start_step_id = 1
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobschedule @job_id=@jobId, @name=N''''Retore log every 10 minutes'''', 
		@enabled=1, 
		@freq_type=4, 
		@freq_interval=1, 
		@freq_subday_type=4, 
		@freq_subday_interval=10, 
		@freq_relative_interval=0, 
		@freq_recurrence_factor=0, 
		@active_start_date=20211115, 
		@active_end_date=99991231, 
		@active_start_time=0, 
		@active_end_time=235959, 
		@schedule_uid=N''''0a38c61c-cdde-4de7-995c-2edd4ee1beb4''''
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobserver @job_id = @jobId, @server_name = N''''(local)''''
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
COMMIT TRANSACTION
GOTO EndSave
QuitWithRollback:
    IF (@@TRANCOUNT > 0) ROLLBACK TRANSACTION
EndSave:
''

Select @DefinitionText = REPLACE(@DefinitionText, ''$LOGSHIP_EXECUTABLE_PATH$'', Ltrim(Rtrim(ParamValue)))  
From msdb.LogShip.LogShipParams
Where ParamName = ''sqllogship.exe''

execute dbo.sp_executesql @statement = @DefinitionText;
'
Insert into LogShip.LogShipObjects(DefinitionText, OrderIndex)
Values(@DefinitionText, 5)

Set @DefinitionText = N'
BEGIN TRANSACTION
DECLARE @ReturnCode INT
SELECT @ReturnCode = 0

IF NOT EXISTS (SELECT name FROM msdb.dbo.syscategories WHERE name=N''Log Shipping'' AND category_class=1)
BEGIN
EXEC @ReturnCode = msdb.dbo.sp_add_category @class=N''JOB'', @type=N''LOCAL'', @name=N''Log Shipping''
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

END

DECLARE @jobId BINARY(16)
EXEC @ReturnCode =  msdb.dbo.sp_add_job @job_name=N''SoSql_LogShip_Copy_All_DB'', 
		@enabled=0, 
		@notify_level_eventlog=0, 
		@notify_level_email=0, 
		@notify_level_netsend=0, 
		@notify_level_page=0, 
		@delete_level=0, 
		@description=N''Ce job ne fait rien mais doit être créé pour que le Log shipping fonctione'', 
		@category_name=N''Log Shipping'', 
		@owner_login_name=N''sa'', @job_id = @jobId OUTPUT
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N''Do nothing'', 
		@step_id=1, 
		@cmdexec_success_code=0, 
		@on_success_action=1, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N''TSQL'', 
		@command=N''--Do nothing'', 
		@database_name=N''msdb'', 
		@flags=0
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_update_job @job_id = @jobId, @start_step_id = 1
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobserver @job_id = @jobId, @server_name = N''(local)''
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
COMMIT TRANSACTION
GOTO EndSave
QuitWithRollback:
    IF (@@TRANCOUNT > 0) ROLLBACK TRANSACTION
EndSave:
'
Insert into LogShip.LogShipObjects(DefinitionText, OrderIndex)
Values(@DefinitionText, 6)

--Génère le fichier de code SQL pour SQLCMD
--Active CMDSHELL
Declare @xp_cmdshellEnabled int;

Select @BackupShare = ParamValue from LogShip.LogShipParams Where ParamName = 'BackupShare'
Select @SourceServer = ParamValue from LogShip.LogShipParams Where ParamName = 'SourceServer'
Select @TargetServer = ParamValue from LogShip.LogShipParams Where ParamName = 'TargetServer'

SELECT @xp_cmdshellEnabled = CONVERT(INT, ISNULL(value, value_in_use))
FROM  sys.configurations
WHERE  name = 'xp_cmdshell' ;

EXEC sp_configure 'show advanced options', 1;
RECONFIGURE;
EXEC sp_configure 'xp_cmdshell', 1;
RECONFIGURE;

DECLARE @Message varchar(MAX), @Severity int, @State smallint;

Declare @Bcp nvarchar(4000)
	Set @Bcp = 'bcp "Use msdb; Select SqlText from LogShip.v_ObjectsScripts order by OrderIndex" queryout "'+@BackupShare+'\LogShipScript.sql" -e '+@BackupShare+'\LogShipScriptBcpError.txt -c '+@BcpUtf8+' -t; -T -S "'+@SourceServer+'"'
	Begin try
		exec xp_cmdshell @Bcp,no_output
		--Génère la commande SQLCMD.
		Set @Bcp = 'sqlcmd -E -S '+@TargetServer+' -i '+@BackupShare+'\LogShipScript.sql		
		'
		Raiserror('
		
Exécutez la commande SQLCMD ci-dessous pour déployer les objets sur le serveur cible depuis le serveur souce en mode CMD !!!
		', 0, 1) with nowait;
		Raiserror(@Bcp, 0, 1) with nowait;
		Raiserror('Les Warning de type [depends on the missing object] peuvent être ignorés !
		
		
		', 0, 1) with nowait;
	End try
	Begin catch
		Raiserror(@Bcp, 0, 1) with nowait;
		Select @Message = ERROR_MESSAGE(),
		@Severity = ERROR_SEVERITY(),
		@State = ERROR_STATE()
	   Raiserror (@Message, @Severity, @State)
	End catch
--Désactive CMDSHELL
EXEC sp_configure 'show advanced options', 1;
RECONFIGURE;
EXEC sp_configure 'xp_cmdshell', @xp_cmdshellEnabled;
RECONFIGURE;
