Use msdb;
GO
/*
Version: 2.0 du 19/03/2023

Script pour ajouter une nouvelle base de donnes sur le serveur primaire du Log Shipping.
*/
Declare @TargetCompatibilityLevel varchar(50), @DatabaseName sysname;
--Affectation des paramtres
--Begin
Set @TargetCompatibilityLevel = '150';			--Niveau de compatibilit cible
Set @DatabaseName = 'DemoDB10'					--Nom de la base  ajouter
--End

/*===== Ne pas modifier le code ci-dessous ! =====*/

IF OBJECT_ID('tempdb..#dbfreesize') is not null
Begin
	Drop table #dbfreesize
End
CREATE TABLE #dbfreesize(
	[DatabaseName] [nvarchar](128) NULL,
	[Drive] [nvarchar](1) NULL,
	[GroupId] int,
	[FILE_SIZE_MB] [decimal](12, 2) NULL,
	[SPACE_USED_MB] [decimal](12, 2) NULL,
	[FREE_SPACE_MB] [decimal](12, 2) NULL,
	[FREE_SPACE_%] [decimal](12, 2) NULL,
	[NAME] [sysname] NOT NULL,
	[FILENAME] [nvarchar](260) NOT NULL
)


insert into #dbfreesize([DatabaseName], [Drive], [GroupId], [FILE_SIZE_MB], [SPACE_USED_MB], [FREE_SPACE_MB], [FREE_SPACE_%], [NAME], [FILENAME]) 
Exec sp_MSforeachdb N'
Use [?];
SELECT
''?'' as DbName,
 SUBSTRING(a.FILENAME, 1, 1) Drive,
 [GroupId] = groupid,
 [FILE_SIZE_MB] = convert(decimal(12,2),
round(a.size/128.000,2)),
 [SPACE_USED_MB] = convert(decimal(12,2),
round(fileproperty(a.name,''SpaceUsed'')/128.000,2)),
 [FREE_SPACE_MB] = convert(decimal(12,2),
round((a.size-fileproperty(a.name,''SpaceUsed''))/128.000,2)) ,
 [FREE_SPACE_%] = convert(decimal(12,2),
(convert(decimal(12,2),round((a.size-fileproperty(a.name,''SpaceUsed''))/128.000,2)) 
/ convert(decimal(12,2),round(a.size/128.000,2)) * 100)),
 a.NAME, a.FILENAME
FROM dbo.sysfiles a
ORDER BY Drive, [Name]'


	IF Not Exists(Select top 1 1 From msdb.LogShip.DatabasesDetails Where [DatabaseName] = @DatabaseName)
	Begin
		Insert into msdb.LogShip.DatabasesDetails([ServerInstanceName], [InstanceName], [DatabaseName], [RecoveryModel], [NewRecoveryModel], [OriginalRecoveryModel], [VersionNum], [NewVersionNum], [CompatibilityLevel])
		SELECT 
		CONVERT(sysname, SERVERPROPERTY('servername')) as ServerInstanceName, CONVERT(sysname, SERVERPROPERTY('instancename')) as InstanceName
		,name AS [DatabaseName],
		recovery_model_desc AS [RecoveryModel],
		Case 
			When recovery_model_desc <> 'FULL' Then 'USE [master]; ALTER DATABASE ['+name+'] SET RECOVERY FULL WITH NO_WAIT' Else NULL 
		End as NewRecoveryModel,
		'USE [master]; ALTER DATABASE ['+name+'] SET RECOVERY '+recovery_model_desc collate French_CI_AS+' WITH NO_WAIT' as OriginalRecoveryModel,
		compatibility_level as VersionNum,
		CONVERT(tinyint, null) as NewVersionNum,
		CompatibilityLevel = 
		CASE compatibility_level
			WHEN 65  THEN 'SQL Server 6.5'
			WHEN 70  THEN 'SQL Server 7.0'
			WHEN 80  THEN 'SQL Server 2000'
			WHEN 90  THEN 'SQL Server 2005'
			WHEN 100 THEN 'SQL Server 2008/R2'
			WHEN 110 THEN 'SQL Server 2012'
			WHEN 120 THEN 'SQL Server 2014'
			WHEN 130 THEN 'SQL Server 2016'
			WHEN 140 THEN 'SQL Server 2017'
			WHEN 150 THEN 'SQL Server 2019'
			ELSE 'new unknown - '+CONVERT(varchar(10),compatibility_level)
		END
		FROM sys.databases
		Where name = @DatabaseName
		And is_read_only = 0

		Update msdb.LogShip.DatabasesDetails
		Set NewVersionNum = @TargetCompatibilityLevel
	End

	IF Not Exists(Select top 1 1 From msdb.LogShip.DbFilesLocation Where [DatabaseName] = @DatabaseName)
	Begin
		Insert into msdb.LogShip.DbFilesLocation([DatabaseName], [Drive], [GroupId], [FILE_SIZE_MB], [SPACE_USED_MB], [FREE_SPACE_MB], [FREE_SPACE_%], [NAME], [FILENAME], [CurrentPath], [NewPath])
		Select *
		,Replace(FILENAME, Reverse(LEFT(REVERSE(FILENAME), Charindex('\', REVERSE(FILENAME))-1)), '') as CurrentPath
		,CONVERT(varchar(4000), null) as NewPath
		from #dbfreesize
		Where [DatabaseName] = @DatabaseName
		Order by [FILE_SIZE_MB] desc
	End

	--Code de lancement
	Raiserror('Pour lancer le Log Shipping sur la base de donnes [%s], excutez le code ci-dessous.', 0, 1, @DatabaseName) with nowait;
	Raiserror('Exec msdb.LogShip.up_Set_LogShipping_ON @LSDatabaseName = ''%s''', 0, 1, @DatabaseName) with nowait;


	